#!/usr/bin/env python
"""
Module ShutdownReadWrite
Sub-Package IO.DATA of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ShutdownReadWrite alternate data
handling class.
"""

import socket


class ShutdownReadWrite(object):
    """Data handling class using socket shutdown.
    
    Mixin read/write class that calls ``shutdown`` on its socket
    when it is finished writing, and detects the end of data
    being read by the read end of the socket closing (which
    is signalled by a read of zero bytes). Obviously, this
    method only works for a single round-trip data exchange.
    Also, this method is only designed to work with sockets;
    hence, the ``shutdown`` method assumes that the socket
    instance variable contains a socket.
    
    Note that the ``auto_close`` class field here is set to
    ``False`` (which means that this class must come *before*
    the ``BaseData`` class in the MRO), so that the channel will
    not automatically be closed when a read of zero bytes is
    detected. This is so that the ``shutdown`` method strategy
    described above will work properly (since it requires that
    each endpoint shut itself down separately, instead of the
    entire channel being closed at once).
    """
    
    auto_close = False
    write_started = False
    
    def read_complete(self):
        # The shutdown_received flag is set by BaseData, which
        # must be somewhere earlier in the MRO
        return self.shutdown_received
    
    def handle_write(self):
        self.write_started = True
        super(ShutdownReadWrite, self).handle_write()
    
    def clear_write(self):
        super(ShutdownReadWrite, self).clear_write()
        if self.write_started:
            self.shutdown()
            self.write_started = False
    
    def shutdown(self):
        if self.shutdown_received:
            # Calling shutdown when the other end already has
            # can lead to undefined behavior; safer to just
            # close instead
            self.close()
        else:
            self.socket.shutdown(socket.SHUT_WR)



#!/usr/bin/env python
"""
Sub-Package TEST.IO of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package contains the PLIB.IO test suite.
"""



#!/usr/bin/env python
"""
TEST.IO.TEST_ERR.PY -- test script for sub-package IO of package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests checking the handling of various
errors or unusual conditions in the I/O modules in the
PLIB.IO sub-package.
"""

import signal
import unittest

# Only run these tests when SIGCHLD is available

if hasattr(signal, 'SIGCHLD'):
    
    import os
    from errno import ECHILD
    
    # For use in hack to get exception info in server exit code below
    import exceptions
    
    from plib.io.mixins import SigChldServerMixin, SigIntServerMixin
    
    from .testlib import IOChannelTest
    from .testlib_blocking import BlockingHandler, ForkingServer
    from .testlib_blocking import SocketClient as BlockingClient
    
    
    class EarlyShutdownClient(BlockingClient):
        # Simple client just records received data
        
        keep_alive = True  # so the channel will stay open after data is read
        
        def process_data(self):
            self.result = self.read_data
    
    
    class EarlyShutdownHandler(BlockingHandler):
        pass  # keep_alive is true by default for server-side communicators
    
    
    # Hack to get exception info on server shutdown
    
    exception_names = dir(exceptions)
    
    def exception_number(e):
        return exception_names.index(e.__class__.__name__) + 130
    
    
    class EarlyShutdownServer(SigChldServerMixin,
                              SigIntServerMixin,
                              ForkingServer):
        # Traps SIGTERM, so we can trigger a shutdown while children are
        # active and confirm that the server shuts them down gracefully
        
        term_sigs = [signal.SIGTERM]
        
        child_was_open = False
        
        def server_close(self):
            # Confirm that a child is still open
            self.child_was_open = bool(self.active_children)
            super(EarlyShutdownServer, self).server_close()
        
        def serve_forever(self):
            # Wrap the super call just in case an error happens in here,
            # so we don't mistake it for an error in the child code below
            try:
                super(EarlyShutdownServer, self).serve_forever()
            except Exception, e:
                # Hack to get some info on which exception it was (see
                # note below on range of exit codes used)
                os._exit(exception_number(e))
            
            # We signal our unique errors with codes that are higher than
            # any possible errno value, but lower than the range PLIB uses
            # for forking errors, so we can tell which error we got
            if not self.child_was_open:
                # The child shut down early
                os._exit(198)
            if self.active_children:
                # The shutdown process didn't clear active_children
                os._exit(197)
            # If we get to here, there was a child and it's no longer in
            # active_children, so we should be home free, but we're
            # paranoid so we check further
            try:
                pid, status = os.waitpid(-1, os.WNOHANG)  # should raise OSError
            except OSError, e:
                os._exit(e.errno)  # hopefully this equals ECHILD!
            # If we get here, there were children, bad!
            if pid:
                if os.WIFSIGNALED(status):
                    os._exit(-os.WTERMSIG(status))
                elif os.WIFEXITED(status):
                    os._exit(os.WEXITSTATUS(status))
            # Just in case something went awry above
            os._exit(199)
    
    
    class BlockingEarlyShutdownTest(IOChannelTest, unittest.TestCase):
        
        client_class = EarlyShutdownClient
        handler_class = EarlyShutdownHandler
        server_class = EarlyShutdownServer
        
        test_data = "Python rocks!"
        
        def test_shutdown(self):
            client = self.client_class()
            # This will do one round-trip exchange but keep the channel open
            client.client_communicate(self.test_data, ('localhost', self.server_port))
            self.assertTrue(not client.closed)
            self.assertEqual(client.result, self.test_data)
            # Now shut down the server while the handler is still active
            self.server.stop()
            exitcode = self.server.exitcode()
            self.server = None
            self.assertEqual(exitcode, ECHILD)
            client.close()
            self.assertTrue(client.closed)


if __name__ == '__main__':
    if not hasattr(signal, 'SIGCHLD'):
        print "SIGCHLD not available, skipping related tests."
    unittest.main()



#!/usr/bin/env python
"""
Module BaseClient
Sub-Package IO.SOCKET of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the socket BaseClient class.
"""

from plib.io.socket import ConnectMixin


class BaseClient(ConnectMixin):
    """Base class for socket clients.
    """
    
    def setup_client(self, addr):
        """Connect to server at addr.
        
        Note that we don't try to connect if we're already
        connected; this means that if we're connected but
        the ``addr`` parameter does not match the address
        we're currently connected to, the new ``addr`` is
        silently ignored and the original connection remains.
        """
        if not self.addr:
            self.do_connect(addr)



#!/usr/bin/env python
"""
Module SerialClientMixin
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous SerialClientMixin class.
"""

from plib.io.serial import BaseClient
from plib.io.async import ClientMixin


class SerialClientMixin(BaseClient, ClientMixin):
    """Mixin class for async serial client.
    
    Call the ``client_communicate method`` to open a serial
    device and send data; override the ``process_data`` method
    to do something with the reply.
    """
    pass



#!/usr/bin/env python
"""
Module AsyncConnectMixin
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous AsyncConnectMixin class.
"""


class AsyncConnectMixin(object):
    """Mixin class to enable async client connect functionality.
    
    Overrides the ``writable`` and ``readable`` methods to return
    ``True`` if a connect is pending (to make sure we get notified
    when the asynchronous connect completes; it could be either a
    read or a write ``select()`` return so we allow for both to
    be safe).
    """
    
    def writable(self):
        return self.connect_pending or \
            super(AsyncConnectMixin, self).writable()
    
    def readable(self):
        return self.connect_pending or \
            super(AsyncConnectMixin, self).readable()



#!/usr/bin/env python
"""
TEST.IO.TEST_PERSISTENT.PY -- test script for sub-package IO of package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the persistent async I/O
classes in the PLIB.IO sub-package.
"""

import unittest
import select

from plib.io import async

from .testlib import PersistentTest
from .testlib_persistent import ReadWriteTestMixin, TerminatorTestMixin

# Only run these tests if poll is available

if hasattr(select, 'poll'):
    
    async.use_poll(True)  # ensure we use poll (not strictly necessary
                          # since True is the default, just shows usage)
    
    
    class XPersistentTest(ReadWriteTestMixin, PersistentTest, unittest.TestCase):
        
        client_list = ["Python rocks!", "Try it today!", "You'll be glad you did!"]
        server_list = ["You betcha!", "It's *much* better than Perl!", "And don't even *mention* C++!"]
    
    
    class XPersistentTestWithPush(XPersistentTest):
        
        test_auto_push = True
    
    
    class XPersistentTestLargeMessage1(XPersistentTest):
        
        client_list = ["a" * 6000, "b" * 6000, "c" * 6000]
        server_list = ["1" * 6000, "2" * 6000, "3" * 6000]
    
    
    class XPersistentTestLargeMessage2(XPersistentTest):
        
        client_list = ["a" * 10000, "b" * 10000, "c" * 10000]
        server_list = ["1" * 10000, "2" * 10000, "3" * 10000]
    
    
    class YPersistentTest(TerminatorTestMixin, PersistentTest, unittest.TestCase):
        
        client_list = ["Python rocks!", "Try it today!", "You'll be glad you did!"]
        server_list = ["You betcha!", "It's *much* better than Perl!", "And don't even *mention* C++!"]
    
    
    class YPersistentTestWithPush(YPersistentTest):
        
        test_auto_push = True
    
    
    class YPersistentTestLargeMessage1(YPersistentTest):
        
        client_list = ["a" * 6000, "b" * 6000, "c" * 6000]
        server_list = ["1" * 6000, "2" * 6000, "3" * 6000]
    
    
    class YPersistentTestLargeMessage2(YPersistentTest):
        
        client_list = ["a" * 10000, "b" * 10000, "c" * 10000]
        server_list = ["1" * 10000, "2" * 10000, "3" * 10000]


if __name__ == '__main__':
    if not hasattr(select, 'poll'):
        print "Poll not available, skipping persistent I/O tests for poll."
    unittest.main()



#!/usr/bin/env python
"""
Module ConnectMixin
Sub-Package IO.SOCKET of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the socket ConnectMixin class.
"""

import socket


class ConnectMixin(object):
    """Mixin class to add socket connect functionality.
    """
    
    address_family = socket.AF_INET
    socket_type = socket.SOCK_STREAM
    
    def do_connect(self, addr):
        """Convenience method to create socket and connect to addr.
        """
        self.create_socket(self.address_family, self.socket_type)
        self.connect(addr)



#!/usr/bin/env python
"""
Module PServerBase
Sub-Package IO.CLASSES of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the PServerBase class, which can be used as
a mixin for any server that conforms to the API of the I/O server
classes in PLIB.STDLIB. This mixin class adds general signal
handling and logging facilities and redirection of the standard
file descriptors, if required. Note that this class does *not*
do anything special to "daemonize" itself or decouple itself from
its parent environment; all it does is set its root directory to
something sane, and redirect I/O if desired. (On any Unix where
a shell script can background a process with the & parameter,
there doesn't seem to be much point in having each daemon program
do the "double fork" itself, not to mention worrying about process
groups, controlling terminals, etc.--since the shell script is
effectively the first fork, and the backgrounding with & is the
second (and also decouples from the shell's tty), what's left?)
"""

import sys
import os
import datetime
import logging
import signal

from plib.io.mixins import SigIntServerMixin

# We include SIGINT here because it appears to get masked when a script
# is backgrounded, we want to unmask it just in case (we could go back
# to the default Python handler that raises KeyboardInterrupt, but it's
# easier and neater just to trap it ourselves)

sigdata = {
    'SIGABRT': ("aborted", 4),
    'SIGHUP': ("hangup", 1),
    'SIGINT': ("interrupt", 3),
    'SIGQUIT': ("quit", 2),
    'SIGTERM': ("terminated", 0),
    None: ("unknown shutdown", -1)
}

for signame in sigdata.iterkeys():
    if signame is not None:
        # Allows for systems where some signals don't exist; those signals
        # will simply be non-functional
        sig = getattr(signal, signame, 0)
        setattr(sys.modules[__name__], signame, sig)


def sigdata_dict(index):
    result = {}
    for key, value in sigdata.iteritems():
        if key is not None:
            sig = getattr(sys.modules[__name__], key)
        else:
            sig = key
        if sig != 0:
            result[sig] = value[index]
    return result


def sigdata_list():
    result = []
    for key in sigdata.iterkeys():
        if key is not None:
            sig = getattr(sys.modules[__name__], key)
            if sig != 0:
                result.append(sig)
    return result


class PServerBase(SigIntServerMixin):
    """Server mixin class providing enhanced common functionality.
    
    Generic base server mixin class, can be used with any type of server
    (sync, async, forking). Implements signal handling for controlled
    termination, and log file functionality. The intent is to trap any
    signal that might be used to indicate general 'program shutdown' as
    opposed to some specific error condition (i.e., any signal where it
    can be assumed that controlled shutdown of the Python interpreter
    is possible).
    """
    
    sig_msgs = sigdata_dict(0)
    ret_codes = sigdata_dict(1)
    term_sigs = sigdata_list()
    sig_methods = {}
    
    terminate_sig = None
    
    if os.name == 'posix':
        log_root = os.path.expanduser("~")
        dir_root = "/"
        log_namestr = ".{name}/{name}.log"
    else:
        log_root = os.getenv("HOME")
        dir_root = log_root
        log_namestr = "{name}.log"
    log_str = "{} {}, time {}"
    server_name = "server"
    
    # This can be set to False for debugging purposes, to allow log entries to
    # be printed to the terminal; if may also be useful if the server is being
    # run by another master process that handles stdin and stdout for it.
    
    redirect_files = True
    
    bind_addr = ("localhost", 9999)
    handler_class = None
    
    def __init__(self, server_addr=None, handler_class=None):
        if server_addr is None:
            server_addr = self.bind_addr
        if handler_class is None:
            handler_class = self.handler_class
        super(PServerBase, self).__init__(server_addr, handler_class)
    
    def server_start(self):
        os.chdir(self.dir_root)
        
        self.logger = self.init_logging()
        
        if self.redirect_files:
            self.log_msg("redirecting standard file descriptors")
            
            if os.name == 'posix':
                # On POSIX systems we can just read/write to /dev/null
                devnull_path = os.path.join("/dev", "null")
                dev_null = open
            else:
                # Hack to emulate a file-like object that does no I/O
                class dev_null(file):
                    
                    def read(self, *args):
                        # Nothing to read
                        return ""
                    
                    readline = read
                    
                    def readlines(self, *args):
                        return []
                    
                    def write(self, *args):
                        # Force this to be a no-op
                        pass
                    
                    writelines = truncate = write
                    
                    def flush(self):
                        # Force this to be a no-op
                        pass
                
                # We still need a real file so the object has a real fileno
                devnull_path = os.path.join(
                    self.dir_root, "{}_devnull".format(self.server_name))
                open(devnull_path, 'w').close()  # creates a 0-byte file
            
            self.dev_null_r = dev_null(devnull_path, 'r')
            self.dev_null_w = dev_null(devnull_path, 'w')
            sys.stdout.flush()
            sys.stderr.flush()
            os.dup2(self.dev_null_r.fileno(), sys.stdin.fileno())
            os.dup2(self.dev_null_w.fileno(), sys.stdout.fileno())
            os.dup2(self.dev_null_w.fileno(), sys.stderr.fileno())
        
        super(PServerBase, self).server_start()
        
        self.log_msg("started")
    
    def init_logging(self):
        log_filename = os.path.join(
            self.log_root, self.log_namestr.format(name=self.server_name))
        logging.basicConfig(filename=log_filename, level=logging.INFO)
        return logging.getLogger()
    
    def log_msg(self, msg, err=False):
        log_entry = self.log_str.format(
            self.server_name, msg, datetime.datetime.now())
        getattr(self.logger, ('info', 'error')[err])(log_entry)
        if not self.redirect_files:
            print log_entry
    
    def handle_error(self):
        self.log_msg("exception occurred", err=True)
        from cStringIO import StringIO
        import traceback
        s = StringIO()
        traceback.print_exc(file=s)
        self.log_msg(s.getvalue(), err=True)
        s.close()
    
    def ret_code(self):
        return self.ret_codes[self.terminate_sig]
    
    def term_sig_handler(self, sig, frame=None):
        """Add ability to specify a dispatch method for sig.
        """
        
        # This sets the terminate flag
        super(PServerBase, self).term_sig_handler(sig, frame)
        
        # Store the signal for logging on shutdown
        self.terminate_sig = sig
        
        # Dispatch to method if present
        if sig in self.sig_methods:
            fn = getattr(self, self.sig_methods[sig])
            fn(sig, frame)
    
    def server_close(self):
        super(PServerBase, self).server_close()
        
        self.log_msg(self.sig_msgs[self.terminate_sig])
        
        if self.redirect_files:
            self.dev_null_w.close()
            self.dev_null_r.close()
            if os.name != 'posix':
                # Undo the /dev/null stdin emulation hack
                devnull_path = os.path.join(
                    self.dir_root, "{}_devnull".format(self.server_name))
                os.remove(devnull_path)
        
        logging.shutdown()



#!/usr/bin/env python
"""
Module PAsyncServer
Sub-Package IO.CLASSES of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``PAsyncServer class``. This is an
asynchronous socket server that includes the general signal
handling and logging facilities of ``PServerBase``.
"""

from plib.io.async import BaseRequestHandler, SocketServer

from .PServerBase import PServerBase


class PAsyncServer(PServerBase, SocketServer):
    """Generic async server class with enhanced common functionality.
    
    This generic async server implements signal handling for controlled
    termination, and log file functionality. The intent is to trap any
    signal that might be used to indicate general 'program shutdown' as
    opposed to some specific error condition (i.e., any signal where it
    can be assumed that controlled shutdown of the Python interpreter
    is possible).
    """
    
    handler_class = BaseRequestHandler



#!/usr/bin/env python
"""
Module UTILS -- I/O class utilities
Sub-Package IO of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains utility functions and classes for
use by the I/O sub-packages. The main ones intended
for external use are:

- Class ``IOModuleProxy``: implements the machinery for
  constructing I/O classes with alternate read/write
  handling on the fly. See the ``plib.io``
  sub-package docstring for an overview of this topic.

- function ``io_class``: the underlying function used by
  ``IOModuleProxy`` to work its magic; it is exposed so
  that, if all else fails, you can call the machinery
  directly.

- global variable ``io_meta``: the metaclass used to
  construct the I/O classes on the fly. It should be very,
  very rare that you need to change this from its default
  of ``type``.
"""

from itertools import chain

from plib.stdlib.util import ModuleProxy
from plib.stdlib.imp import import_from_module


# Machinery to add the "standard" I/O mixin classes to the
# async and blocking I/O sub-package namespaces

def modname_from_pkgname(pkgname):
    return pkgname.split('.')[-1]


def format_doc(doc, pkgname):
    if '{}' in doc:
        return doc.format(modname_from_pkgname(pkgname))
    return doc


def mixin_helper(pkgname, klassname, bases, doc):
    def f():
        baselist = [
            import_from_module(pkgname, basename)
            for basename in bases
        ]
        return type(klassname, tuple(baselist), {
            '__module__': pkgname,
            '__doc__': format_doc(doc, pkgname)
        })
    f.__name__ = "{}_{}_helper".format(modname_from_pkgname(pkgname), klassname)
    return f


_BaseRequestHandler_doc = """
    Basic {} request handler; default is to do one
    round-trip exchange of data and then shut down.
    """

_SerialClient_doc = """
    Basic {} serial device client class. Call the
    ``client_communicate`` method to open a serial device
    and send data; override the ``process_data`` method to
    do something with the reply.
    """

_SerialServer_doc = """
    Basic {} serial device server class. Call the
    ``serve_forever`` method to run the server. Note that the
    default is to remain open for an unlimited number of
    round-trip data exchanges; override the ``query_done`` method
    to determine when the server should close.
    """

_SocketClient_doc = """
    Basic {} socket client class. Call the
    ``client_communicate`` method to connect to a server
    and send data; override the ``process_data`` method to
    do something with the reply.
    """

_mixin_names = [
    ('BaseRequestHandler', ('ServerMixin', 'RequestBase'), _BaseRequestHandler_doc),
    ('SerialClient', ('SerialClientMixin', 'SerialBase'), _SerialClient_doc),
    ('SerialServer', ('SerialServerMixin', 'SerialBase'), _SerialServer_doc),
    ('SocketClient', ('SocketClientMixin', 'SocketBase'), _SocketClient_doc)
]

_PersistentRequestHandler_doc = """
    Base class for persistent, full-duplex asynchronous socket
    request handler.
    """

_PersistentSerial_doc = """
    Base class for persistent, full-duplex asynchronous serial
    device I/O. Can be used for both clients and servers.
    """

_PersistentSocket_doc = """
    Base class for persistent, full-duplex asynchronous socket
    I/O. Can be used for both clients and servers, but intended
    mainly for clients that need connect functionality. (For
    server-side persistent sockets, you should normally use the
    ``PersistentRequestHandler`` class with ``SocketServer``.)
    """

_persistent_names = [
    ('PersistentRequestHandler', ('PersistentMixin', 'RequestBase'), _PersistentRequestHandler_doc),
    ('PersistentSerial', ('PersistentMixin', 'SerialBase'), _PersistentSerial_doc),
    ('PersistentSocket', ('PersistentSocketMixin', 'SocketBase'), _PersistentSocket_doc)
]


def _get_names(add_persistent):
    """ Return the list of mixin class name entries to be used. """
    
    if add_persistent:
        return chain(_mixin_names, _persistent_names)
    return _mixin_names


def _get_mixin_names(pkgname, add_persistent):
    """Return "standard" mixin entries for I/O classes in pkgname.
    
    This means we don't have to have "boilerplate" .py module files
    in sub-packages for classes that are simple mixins.
    """
    
    return dict(
        (klassname, mixin_helper(pkgname, klassname, bases, doc))
        for klassname, bases, doc in _get_names(add_persistent)
    )


# Specialized ModuleProxy that can generate alternate read/write
# classes on the fly; this allows you to append 'WithShutdown',
# 'WithTerminator', or 'WithReadWrite' to a valid class name from
# above and have the appropriate read/write handling class spliced
# into the base class list for the class you've requested. This is
# nice because now you don't have to remember exactly where in the
# MRO the read/write class has to go to work properly; that's all
# done automatically. (Note that we allow for the possibility--very
# unlikely, we hope--that someone might want to use a custom
# metaclass to construct these classes; hence the io_meta global
# and the meta parameter to io_class.)

_readwrite_doc = """
    Uses the {} data handling mixin class.
    """

_rwbases = {
    'WithShutdown': 'ShutdownReadWrite',
    'WithTerminator': 'TerminatorReadWrite',
    'WithReadWrite': 'ReadWrite'
}

io_meta = type


def io_class(pkgname, name, rwbase, add_persistent=False, endstr="", meta=None):
    """
    Return a modified version of class ``name`` from package ``pkgname``
    with alternate read/write class ``rwbase`` spliced into its base
    class list. If ``add_persistent``, include the "persistent" class
    names in the validity check for ``name``; if ``endstr``, append
    that string to the name of the created class (instead of the default
    based on ``rwbase.__name__``); if ``meta`` is not ``None``, use the
    given metaclass to create the class to be returned (instead of the
    default stored in the ``io_meta`` global).
    """
    
    for klassname, bases, doc in _get_names(add_persistent):
        if klassname == name:
            base1, base2 = [
                import_from_module(pkgname, base)
                for base in bases
            ]
            if not endstr:
                endstr = "With{}".format(rwbase.__name__)
            rwdoc = _readwrite_doc.format(rwbase.__name__)
            if meta is None:
                meta = io_meta
            return meta(klassname + endstr,
                        (base1, rwbase, base2),
                        {
                            '__module__': pkgname,
                            '__doc__': format_doc(doc + rwdoc, pkgname)
                        })
    raise ValueError("IO class {} not found".format(name))


class IOModuleProxy(ModuleProxy):
    # ``ModuleProxy`` subclass that adds the ability to construct
    # alternate read/write classes on the fly, so that they can
    # appear in the module namespace.
    
    def init_proxy(self, pkgname, path, globals_dict, locals_dict,
                   names=None, excludes=None, autodiscover=True,
                   add_persistent=False, meta=None):
        
        self._mixin_names = _get_mixin_names(pkgname, add_persistent)
        self._rw_names = dict(
            ('{}{}'.format(basename, rwsuffix), rwsuffix)
            for basename in self._mixin_names
            for rwsuffix in _rwbases.keys()
        )
        ModuleProxy.init_proxy(self, pkgname, path,
                               globals_dict, locals_dict,
                               names=names, excludes=excludes,
                               autodiscover=autodiscover)
        self.add_persistent = add_persistent
        self.meta = meta
    
    def __dir__(self):
        return sorted(dir(self._mod) + self._realized_names
                      + list(self._names.iterkeys())
                      + list(self._mixin_names.iterkeys())
                      + list(self._rw_names.iterkeys()))
    
    def _get_name(self, name):
        try:
            return ModuleProxy._get_name(self, name)
        except KeyError:
            mnames = self._mixin_names
            rwnames = self._rw_names
            
            if name in mnames:
                result = mnames[name]()  # we know it's a helper so just call it
                self._realize_name(name, result)
                del mnames[name]
            
            elif name in rwnames:
                endstr = rwnames[name]
                assert name.endswith(endstr)
                basename = name[:-len(endstr)]
                
                rwbase = import_from_module(
                    'plib.io.data', _rwbases[endstr])
                result = self.get_readwrite_class(
                    basename, rwbase, endstr)
                
                self._realize_name(name, result)
                del rwnames[name]
        
        return result
    
    def get_readwrite_class(self, basename, rwklass, endstr=""):
        """
        Returns an alternate to valid I/O class ``basename`` with the
        read/write handling class ``rwklass`` spliced into its MRO. This
        allows access to the same machinery as ``IOModuleProxy`` uses
        for alternate read/write handling classes.
        """
        
        return io_class(self._mod.__name__, basename, rwklass,
                        self.add_persistent, endstr, self.meta)



#!/usr/bin/env python
"""
Module ReadWriteMonitor
Sub-Package IO.CLASSES of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ReadWriteMonitor class. This is
a useful class for testing client/server I/O channels; it
prints notification of all significant read/write method
calls to standard output, along with diagnostic info on
important state variables.
"""


class ReadWriteMonitor(object):
    
    def _delimiter(self):
        print "-" * 40
    
    def handle_connect(self):
        print "{} handling connect".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).handle_connect()
    
    def readable(self):
        result = super(ReadWriteMonitor, self).readable()
        print "{} readable: {}".format(self.__class__.__name__, result)
        return result
    
    def _read_data_monitor(self):
        print "{} read data {!r}".format(
            self.__class__.__name__, self.read_data)
        print "{} shutdown received: {}".format(
            self.__class__.__name__, self.shutdown_received)
        print "{} channel closed: {}".format(
            self.__class__.__name__, self.channel_closed())
    
    def _read_flag_monitor(self):
        if hasattr(self, 'terminator_received'):
            print "{} terminator received: {}".format(
                self.__class__.__name__, self.terminator_received)
        if hasattr(self, 'read_len'):
            print "{} bytes expected: {}".format(
                self.__class__.__name__, self.read_len)
        if hasattr(self, 'bytes_read'):
            print "{} bytes read: {}".format(
                self.__class__.__name__, self.bytes_read)
    
    def handle_read(self):
        self._delimiter()
        self._read_data_monitor()
        self._read_flag_monitor()
        print "{} handling read".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).handle_read()
        self._read_data_monitor()
        self._read_flag_monitor()
        self._delimiter()
    
    def read_complete(self):
        result = super(ReadWriteMonitor, self).read_complete()
        print "{} read complete: {}".format(
            self.__class__.__name__, result)
        return result
    
    def process_data(self):
        print "{} processing data {!r}".format(
            self.__class__.__name__, self.read_data)
        super(ReadWriteMonitor, self).process_data()
    
    def clear_read(self):
        self._delimiter()
        self._read_flag_monitor()
        print "{} clearing read data".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).clear_read()
        self._read_flag_monitor()
        self._delimiter()
    
    def start(self, data):
        print "{} starting data {!r}".format(
            self.__class__.__name__, data)
        super(ReadWriteMonitor, self).start(data)
    
    def writable(self):
        result = super(ReadWriteMonitor, self).writable()
        print "{} writable: {}".format(
            self.__class__.__name__, result)
        return result
    
    def _write_data_monitor(self):
        print "{} write data {!r}".format(
            self.__class__.__name__, self.write_data)
    
    def _write_flag_monitor(self):
        if hasattr(self, 'terminator_written'):
            print "{} terminator written: {}".format(
                self.__class__.__name__, self.terminator_written)
        if hasattr(self, 'formatted'):
            print "{} formatted: {}".format(
                self.__class__.__name__, self.formatted)
    
    def handle_write(self):
        self._delimiter()
        self._write_data_monitor()
        self._write_flag_monitor()
        print "{} handling write".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).handle_write()
        self._write_data_monitor()
        self._write_flag_monitor()
        self._delimiter()
    
    def write_complete(self):
        result = super(ReadWriteMonitor, self).write_complete()
        print "{} write complete: {}".format(
            self.__class__.__name__, result)
        return result
    
    def clear_write(self):
        self._delimiter()
        self._write_flag_monitor()
        print "{} clearing write data".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).clear_write()
        self._write_flag_monitor()
        self._delimiter()
    
    def check_done(self):
        print "{} done: {}".format(
            self.__class__.__name__, self.done)
        super(ReadWriteMonitor, self).check_done()
        print "{} done: {}".format(
            self.__class__.__name__, self.done)
    
    def handle_error(self):
        print "{} handling error".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).handle_error()
    
    def handle_close(self):
        print "{} handling close".format(self.__class__.__name__)
        super(ReadWriteMonitor, self).handle_close()



#!/usr/bin/env python
"""
Module SocketServer
Sub-Package IO.BLOCKING of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains a simplified alternate to the SocketServer
class from the Python standard library. This class is not very
useful in itself (like the Python standard library equivalent)
because it can only handle one request at a time; but it also
serves as a common base class for the more useful blocking
socket servers, ``ForkingServer`` and ``ThreadingServer``.
All three of these classes, collectively, are discussed in
the following.

The key changes from the Python standard library are:

- Simpler method structure (based on the general I/O methods
  used by all modules in this sub-package)
  
- Only TCP INET servers are included, since these classes
  are intended to be fully portable (so no Unix sockets are
  included), and since UDP support is difficult to get working
  on Linux.
  
- The forking server by default sets its SIGCHLD handler to
  SIG_IGN (on platforms that have SIGCHLD). This means the
  kernel will take care of reaping zombie children when they
  exit, so our code doesn't have to worry about it. If more
  control over child processes is needed, a mixin class is
  available, ``SigChldServerMixin``, which adds SIGCHLD
  handling functionality.
  
- It is possible to set a timeout while listening for requests
  (as is done in Python 2.6 and later), but there is none by
  default, since one shouldn't be needed. As far as I can tell,
  the reason for adding the timeout functionality in Python
  2.6 was to support PyGTK, but I have a hard time seeing why
  you'd want to run a server in the same process as a GUI event
  loop. Servers are *supposed* to be run in their own processes;
  a GUI to make use of the server's functionality should be a
  *client*, talking to the server through a socket. (And if you
  want to do that with a client, the client should be using
  asynchronous I/O anyway--see the client classes in the
  plib.io.async sub-package.)
  
- No attempt is made to allow the server to be shut down by a
  method call from another thread. The only place multiple
  threads are used in this library is the threading server, and
  the only threads it spawns are request handlers, which should
  not be able to shut down the server with a method call. In
  any case, servers shouldn't be running in processes which have
  multiple threads doing other things that might shut the server
  process down; those things should be done in their own
  separate processes, talking to the server through a socket...
"""

import select
import socket
from errno import EINTR
from functools import partial

try:
    from errno import ERESTART  # won't work on OS X
except ImportError:
    ERESTART = None  # safe alternate

from plib.io.mixins import SelfPipeServerMixin
from plib.io.socket import BaseServer, SocketIOBase


# We make a partial of this for the keep_running check if a
# callback is passed to serve_forever

def keep_running_callback(self, callback):
    c = callback()
    return self.keep_running() and (c is not False)


class SocketServer(SelfPipeServerMixin, BaseServer, SocketIOBase):
    """Blocking TCP server base class.
    
    Uses the self-pipe trick to be notified of signals that
    should break out of the ``serve_forever`` loop. The select
    call is added to ``server_loop`` to facilitate this. A
    timeout can be specified with the ``poll_timeout`` class
    field, but there is none by default (which is desirable
    because of the inefficiency of cycling the loop when nothing
    is happening).
    """
    
    poll_timeout = None
    
    def __init__(self, server_addr, handler_class):
        SocketIOBase.__init__(self)
        BaseServer.__init__(self, server_addr, handler_class)
    
    def start_child(self, handler, conn, addr):
        """Start a "child" to handle a request.
        
        This default implementation simply runs the handler
        synchronously in the same process. Derived classes
        override to either fork a new process or start a new
        thread to run the handler, for concurrency.
        """
        try:
            handler(conn, addr, self)
        except:
            self.handle_error()
    
    def handle_accept(self):
        """Get the request info and start a handler.
        """
        self.on_accept()
        conn, addr = self.accept()
        self.start_child(self.handler, conn, addr)
    
    def on_accept(self):
        """Placeholder for derived classes.
        """
        pass
    
    def handle_error(self):
        self.close()
        raise
    
    def close(self):
        if not self.closed:
            self.server_close()
            self.handle_close()
        super(SocketServer, self).close()
    
    def handle_close(self):
        self.on_close()
    
    def on_close(self):
        """Placeholder for derived classes.
        """
        pass
    
    def server_loop(self, callback=None):
        if callback is not None:
            _keep_running = partial(keep_running_callback,
                                    self, callback)
        else:
            _keep_running = self.keep_running
        while _keep_running():
            try:
                pipe_fd = self.pipe.fileno()
                sock_fd = self.socket.fileno()
                r, w, e = select.select([pipe_fd, sock_fd],
                                        [], [], self.poll_timeout)
                if pipe_fd in r:
                    # This triggers self.signal_callback for each signal
                    self.pipe.receive_signals()
                elif sock_fd in r:
                    # Only try accepting if pipe wasn't read
                    self.handle_accept()
            except (socket.error, select.error), why:
                # If we get an 'interrupted system call', don't shut
                # down, just re-start the call after checking
                # keep_running to see if the interruption was a
                # terminate signal
                if why[0] in (EINTR, ERESTART):
                    continue
                else:
                    self.handle_error()



#!/usr/bin/env python
"""
TEST.IO.TEST_NOPOLL.PY -- test script for sub-package IO of package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the async I/O modules in
the PLIB.IO sub-package, with the poll function disabled
(so that select must be used).
"""

import unittest

from plib.io import async

from .testlib import ClientServerTest, SmallBufferTest
from .testlib_async import *

async.use_poll(False)  # force use of select in all async loops


class NonBlockingSocketTestNoPoll(AsyncTestMixin,
                                  ClientServerTest,
                                  unittest.TestCase):
    pass


class NonBlockingSocketTestNoPollMultipleTrips(NonBlockingSocketTestNoPoll):
    
    number_of_trips = 3


class NonBlockingSocketTestLargeMessage1NoPoll(NonBlockingSocketTestNoPoll):
    
    test_data = "a" * 6000


class NonBlockingSocketTestLargeMessage2NoPoll(NonBlockingSocketTestNoPoll):
    
    test_data = "a" * 10000


class NonBlockingSocketTestBufsizeNoPoll(AsyncShutdownTestMixin,
                                         ClientServerTest,
                                         unittest.TestCase):
    
    test_data = "x" * AsyncShutdownClient.bufsize


class ReadWriteTestNoPoll(AsyncReadWriteTestMixin,
                          ClientServerTest,
                          unittest.TestCase):
    pass


class ReadWriteTestNoPollMultipleTrips(ReadWriteTestNoPoll):
    
    number_of_trips = 3


class ReadWriteTestLargeMessage1NoPoll(ReadWriteTestNoPoll):
    
    test_data = "a" * 6000


class ReadWriteTestLargeMessage2NoPoll(ReadWriteTestNoPoll):
    
    test_data = "a" * 10000


class ReadWriteTestBufsizeNoPoll(ReadWriteTestNoPoll):
    
    # total data including read/write encoding should be exactly one buffer
    test_data = "x" * (
        AsyncReadWriteClient.bufsize
        - len(str(AsyncReadWriteClient.bufsize))
        - len(AsyncReadWriteClient.bufsep)
    )


class ReadWriteTestSmallBufferNoPoll(AsyncReadWriteTestMixin,
                                     SmallBufferTest,
                                     unittest.TestCase):
    pass


class TerminatorTestNoPoll(AsyncTerminatorTestMixin,
                           ClientServerTest,
                           unittest.TestCase):
    pass


class TerminatorTestNoPollMultipleTrips(TerminatorTestNoPoll):
    
    number_of_trips = 3


class TerminatorTestLargeMessage1NoPoll(TerminatorTestNoPoll):
    
    test_data = "a" * 6000


class TerminatorTestLargeMessage2NoPoll(TerminatorTestNoPoll):
    
    test_data = "a" * 10000


class TerminatorTestBufsizeNoPoll(TerminatorTestNoPoll):
    
    # total data including terminator should be exactly one buffer
    test_data = "x" * (
        AsyncTerminatorClient.bufsize
        - len(AsyncTerminatorClient.terminator)
    )


class TerminatorTestSmallBufferNoPoll(AsyncTerminatorTestMixin,
                                      SmallBufferTest,
                                      unittest.TestCase):
    pass


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/env python
"""
Module SerialServerMixin
Sub-Package IO.BLOCKING of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O SerialServerMixin class.
"""

from plib.io.serial import BaseServer
from plib.io.blocking import ServerMixin


class SerialServerMixin(BaseServer, ServerMixin):
    """Mixin class for blocking server-side serial I/O.
    """
    pass



#!/usr/bin/env python
"""
Module SerialClientMixin
Sub-Package IO.BLOCKING of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the blocking I/O SerialClientMixin class.
"""

from plib.io.serial import BaseClient
from plib.io.blocking import ClientMixin


class SerialClientMixin(BaseClient, ClientMixin):
    """Mixin class for client-side blocking serial I/O.
    """
    pass



#!/usr/bin/env python
"""
Module SigChldServerMixin
Sub-Package IO.MIXINS of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``SigChldServerMixin`` class, which
specializes ``SigChldMixin`` for servers conforming to the
PLIB I/O server API. The original use case for this class
is as a mixin for the forking ``SocketServer`` class in
``plib.io.blocking``.
"""

import signal

from plib.stdlib.classes import SigChldMixin


class SigChldServerMixin(SigChldMixin):
    """Mixin class for PLIB servers to properly reap dead children.
    """
    
    ignore_sigchld = False  # Overrides default for ForkingServer
    
    def server_start(self):
        # Override to set up the signal handler
        super(SigChldServerMixin, self).server_start()
        if hasattr(signal, 'SIGCHLD'):
            pipe = getattr(self, 'pipe', None)
            if (pipe is not None):
                # If we are using the self-pipe trick (if so, the
                # SelfPipeServerMixin class should have set up the
                # pipe in the super call above), we use its
                # mechanism to track signals; note that we ask for
                # the signal handler to be reset each time to
                # ensure BSD semantics
                self.pipe.track_signal(signal.SIGCHLD, reset=True)
            else:
                self.setup_child_sig_handler()
    
    def _new_child(self, handler, conn, addr):
        child = super(SigChldServerMixin, self)._new_child(
            handler, conn, addr)
        self.track_child(child)
        return child
    
    def start_child(self, handler, conn, addr):
        """Start a new child process and keep track of it.
        
        Note that we call ``reap_children`` here only as a last
        resort safety measure, in case any are missed by the reaping
        in response to the SIGCHLD handler (or in case we're on a
        platform that doesn't have SIGCHLD--see the module docstring
        for ``SigChldMixin``).
        """
        
        self.reap_children()
        super(SigChldServerMixin, self).start_child(handler, conn, addr)
    
    def signal_callback(self, sig):
        # This is called if the self-pipe trick is being used, once
        # for each signal received; we only handle SIGCHLD here and
        # pass other signals on
        if sig == signal.SIGCHLD:
            self.reap_children()
        else:
            super(SigChldServerMixin, self).signal_callback(sig)
    
    def server_close(self):
        """Terminate all children when the server closes.
        """
        self.close_children()
        super(SigChldServerMixin, self).server_close()



#!/usr/bin/env python
"""
Module PAsyncClient
Sub-Package IO.CLASSES of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``PAsyncClient`` class. This is an
asynchronous socket I/O client using the ``PClientBase``
interface.
"""

from plib.io.async import SocketClient
from plib.io.classes import PClientBase


class PAsyncClient(PClientBase, SocketClient):
    """Asynchronous TCP client class.
    
    Connects with TCP server, writes data, and stores the
    result. See the ``PClientBase`` docstring for more
    information.
    """
    pass



#!/usr/bin/env python
"""
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package includes classes for handling asynchronous
I/O, including both serial and socket I/O channels. See
the docstring for the parent package, ``plib.io``,
for information on how this package fits into the overall
I/O package structure.
"""

from plib.io.utils import IOModuleProxy

from ._async import *

IOModuleProxy(__name__).init_proxy(__name__, __path__,
                                   globals(), locals(),
                                   add_persistent=True)

# Now clean up our namespace
del IOModuleProxy



#!/usr/bin/env python
"""
Module SelfPipeServerMixin
Sub-Package IO.MIXINS of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``SelfPipeServerMixin`` class,
which provides "drop-in" usage of ``SelfPipe`` for
servers that conform to the PLIB I/O server API. The
``SocketServer`` class in ``plib.io.async`` uses
this class to implement the self-pipe trick.
"""

from plib.stdlib.classes import SelfPipe


class SelfPipeServerMixin(object):
    """Implements the self-pipe trick for PLIB servers.
    
    The ``pipe_class`` field stores the class that will be instantiated
    to create the pipe (an example is the async socket server, as
    noted above).
    """
    
    pipe_class = SelfPipe
    
    pipe = None
    
    def signal_callback(self, sig):
        # Override to process signals received when the pipe is triggered
        pass
    
    def server_start(self):
        super(SelfPipeServerMixin, self).server_start()
        self.pipe = self.pipe_class(self.signal_callback)



#!/usr/bin/env python
"""
Module CHAT_REPLIES
Sub-Package IO.CLASSES of Package PLIB -- General Python Utilities
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``chat_replies`` class, which wraps an
asynchronous socket client so it looks like a generator. For
an example of usage, see the ``pyidserver.py`` example program.
"""

from plib.stdlib.coll import fifo

from plib.io.async import SocketClient


class chat_replies(SocketClient):
    """Data exchange of list of items as a generator.
    
    Generator that sends data items to a server one by one
    and yields the replies.
    """
    
    keep_alive = True
    
    def __init__(self, addr, items, callback=None,
                 connect_fn=None, close_fn=None):
        self.server_addr = addr
        self.callback = callback
        self.connect_fn = connect_fn
        self.close_fn = close_fn
        self.data_queue = fifo(items)
        super(chat_replies, self).__init__()
    
    def on_connect(self):
        if self.connect_fn:
            self.connect_fn()
    
    def start(self, data):
        self.reply = None
        if data is not None:
            super(chat_replies, self).start(data)
    
    def process_data(self):
        self.reply = self.read_data
    
    def do_loop(self, callback=None):
        if callback is not None:
            if self.callback is not None:
                c = self.callback
                
                def f():
                    c1 = c()
                    c2 = callback()
                    if (c1 is False) or (c2 is False):
                        return False
                
            else:
                f = callback
        else:
            f = self.callback
        super(chat_replies, self).do_loop(f)
    
    def __iter__(self):
        try:
            self.setup_client(self.server_addr)
            while self.data_queue and not self.closed:
                self.client_communicate(self.data_queue.nextitem())
                if self.reply is None:
                    # If we didn't get any data back, exit the generator
                    break
                else:
                    yield self.reply
        finally:
            self.close()
    
    def on_close(self):
        if self.close_fn:
            self.close_fn()



#!/usr/bin/env python
"""
ECHO_CLIENT.PY
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

Python implementation of an "echo" client, which reads
input from stdin and sends it to an "echo" server that
just sends the same data back. Received data is printed
to stdout.

This class illustrates the use of the ``PClientBase``
class in ``plib.io.classes``, which provides a simple way
of sending requests to servers and getting replies back.
It also illustrates how the ``plib.io`` library
is set up to allow easy switching between different I/O
types (async and blocking)--even, as in this case,
switching at runtime based on a command line argument.
"""

import sys
import os

from plib.io.classes import PClientBase
from plib.stdlib.imp import import_from_module

if (len(sys.argv) > 1):
    iotype = sys.argv[1]
else:
    iotype = 'blocking'

SocketClientWithTerminator = import_from_module(
    'plib.io.{}'.format(iotype), "SocketClientWithTerminator")


class EchoClient(PClientBase, SocketClientWithTerminator):
    """Echo client for testing (something like netcat).
    
    Waits for greeting message when first connecting. Prints all
    received data to stdout. Exits on Ctrl-C or EOF (Ctrl-D),
    or by typing "QUIT".
    """
    
    keep_alive = True
    terminator = os.linesep
    
    server_addr = ("localhost", 7000)
    QUIT = "QUIT"
    
    greeting_received = False
    
    def on_connect(self):
        print "Got connection!"
    
    def process_data(self):
        super(EchoClient, self).process_data()
        if not self.greeting_received:
            self.greeting_received = True
    
    def setup_client(self, client_id):
        super(EchoClient, self).setup_client(client_id)
        self.wait_for(self, 'greeting_received')
    
    def run(self):
        # Run interactive session (the constructor
        # has already connected us)
        try:
            while 1:
                data = raw_input()
                if data == self.QUIT:
                    self.close()
                    break
                print self(data)
        except (KeyboardInterrupt, EOFError):
            self.close()
    
    def on_close(self):
        print "Closing connection!"


if __name__ == '__main__':
    EchoClient().run()



#!/usr/bin/env python
"""
TEST.IO.TEST_CHAT_REPLIES.PY -- test script for chat_replies module
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This script contains unit tests for the chat_replies module of
plib.io.
"""

import unittest

from plib.io.mixins import EchoRequestMixin
from plib.io.async import PersistentRequestHandler, SocketServer
from plib.io.classes import chat_replies

from .testlib import IOServerMixin, IOChannelTest


class ChatHandler(EchoRequestMixin, PersistentRequestHandler):
    pass


class ChatServer(IOServerMixin, SocketServer):
    pass


class ChatTestMixin(object):
    
    handler_class = ChatHandler
    server_class = ChatServer


class ChatClientTest(ChatTestMixin, IOChannelTest, unittest.TestCase):
    
    def test_chat_client(self):
        seq = ["Python rocks!", "Try it today!", "You'll be glad you did!"]
        results = [reply for reply in chat_replies(
            ('localhost', self.server_port), seq)]
        self.assertEqual(results, seq)


def testcallback():
    pass


class ChatClientTestCallback(ChatTestMixin, IOChannelTest, unittest.TestCase):
    
    def test_chat_callback(self):
        seq = ["You betcha!", "It's *much* better than Perl!", "And don't even *mention* C++!"]
        results = [reply for reply in chat_replies(
            ('localhost', self.server_port), seq, testcallback)]
        self.assertEqual(results, seq)


start_msg = "Started!"


class ChatHandlerNone(ChatHandler):
    
    def __init__(self, request, client_address, server):
        ChatHandler.__init__(self, request, client_address, server)
        self.start(start_msg)


class ChatClientTestWithNone(ChatTestMixin, IOChannelTest, unittest.TestCase):
    
    handler_class = ChatHandlerNone
    
    def test_chat_client(self):
        seq = [None, "Python rocks!", "Try it today!", "You'll be glad you did!"]
        results = [reply for reply in chat_replies(
            ('localhost', self.server_port), seq)]
        self.assertEqual(results, [start_msg] + seq[1:])


if __name__ == '__main__':
    unittest.main()



#!/usr/bin/python -u
"""
Setup script for PLIB.IO package
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information
"""

from plib.io import __version__ as version

name = "plib.io"
description = "Flexible handling of I/O channels."

author = "Peter A. Donis"
author_email = "peterdonis@alum.mit.edu"

startline = 5
endspec = "The Zen of PLIB"

dev_status = "Beta"

license = "GPLv2"

data_dirs = ["examples"]

classifiers = """
Environment :: Console
Environment :: MacOS X
Environment :: Win32 (MS Windows)
Intended Audience :: Developers
Operating System :: MacOS :: MacOS X
Operating System :: Microsoft :: Windows
Operating System :: POSIX
Operating System :: POSIX :: Linux
Programming Language :: Python :: 2.7
Topic :: Software Development :: Libraries :: Python Modules
"""

requires = """
plib.stdlib (>=0.9.3)
"""

post_install = ["plib-setup-{}.py".format(s) for s in ("io-examples",)]

rst_header_template = """**{basename}** for {name} {version}

:Author:        {author}
:Release Date:  {releasedate}
"""


if __name__ == '__main__':
    import sys
    import os
    from subprocess import call
    from distutils.core import setup
    from setuputils import convert_rst, current_date, setup_vars
    
    convert_rst(rst_header_template,
        startline=2,
        name=name.upper(),
        version=version,
        author=author,
        releasedate=current_date("%d %b %Y")
    )
    call(['sed', '-i', 's/bitbucket.org\/pdonis\/plib-/pypi.python.org\/pypi\/plib./', 'README'])
    call(['sed', '-i', 's/bitbucket.org\/pdonis\/plib3-/pypi.python.org\/pypi\/plib3./', 'README'])
    call(['sed', '-i', 's/github.com\/pdonis/pypi.python.org\/pypi/', 'README'])
    setup(**setup_vars(globals()))
    if "install" in sys.argv:
        for scriptname in post_install:
            os.system(scriptname)



#!/usr/bin/env python
"""
Module BaseClient
Sub-Package IO.SERIAL of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the serial BaseClient class.
"""


class BaseClient(object):
    """Base class for serial I/O clients.
    """
    
    def setup_client(self, devid):
        """Open the serial device ID.
        """
        if not self.port:
            self.create_port(devid)



#!/usr/bin/env python
"""
Module ChildWrapperMixin
Sub-Package IO.MIXINS of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the ``ChildWrapperMixin`` class, which
implements common functionality for servers conforming to
the PLIB I/O API that create child processes or threads
to handle requests.
"""

from functools import partial


def _child(error_fn, child_fn):
    # Child process/thread function
    try:
        child_fn()
        return 0
    except:
        try:
            error_fn()
        except:
            pass
        return 1


def _errorhandler(request):
    # Error handler that closes request
    try:
        request.close()
    except:
        pass


class ChildWrapperMixin(object):
    """Mixin class for PLIB servers to handle child processes/threads.
    
    Subclasses must fill in the ``wrapper_class`` field with a
    ``ChildWrapper`` subclass which will be used to instantiate each
    request handler process/thread.
    """
    
    wrapper_class = None
    
    def server_start(self):
        super(ChildWrapperMixin, self).server_start()
        # Make sure our request handlers shut down when we do
        self.wrapper_class.shutdown_with_parent = True
    
    def _new_child(self, handler, conn, addr):
        return self.wrapper_class(_child,
                                  partial(_errorhandler, conn),
                                  partial(handler, conn, addr, self))
    
    def start_child(self, handler, conn, addr):
        child = self._new_child(handler, conn, addr)
        child.start()



#!/usr/bin/env python
"""
Module SocketBase
Sub-Package IO.ASYNC of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the asynchronous SocketBase class.
"""

from plib.io.socket import SocketData
from plib.io.async import SocketDispatcher


class SocketBase(SocketData, SocketDispatcher):
    """Base class for socket async I/O.
    
    Socket async I/O class with data handling and defaults
    for events that don't need handlers.
    """
    pass



#!/usr/bin/env python
"""
Sub-Package IO.COMM of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This sub-package includes classes for communication
channels customized for a particular data pattern (client,
server, or persistent).
"""

from plib.stdlib.util import ModuleProxy

ModuleProxy(__name__).init_proxy(__name__, __path__, globals(), locals())

# Now clean up our namespace
del ModuleProxy



#!/usr/bin/env python
"""
Module SETUPUTILS -- Utilities to automate boilerplate in Python setup scripts
Copyright (C) 2012 by Peter A. Donis

Released under the Python Software Foundation License.

The purpose of this module is to help automate away much of the
boilerplate that goes into Python setup scripts. Typical usage:

    # declare variables here, for example...
    
    name = "myprog"
    
    description = "My Python Program"
    
    # other variables depending on what you need, but the autodiscovery
    # capabilities of setuputils remove the need for a lot of manual
    # declarations, or at least make them easier
    
    if __name__ == '__main__':
        from distutils.core import setup
        from setuputils import setup_vars
        setup(**setup_vars(globals()))

Or, you can take more fine-grained control over things by only
invoking particular sub-functions, for example:

    if __name__ == '__main__':
        from distutils.core import setup
        from setuputils import autodiscover_packages
        setup(
            name="myprog",
            description="My Python Program",
            packages=autodiscover_packages(globals()),
            # other args
        )

You can even mix the two methods, disabling general autodiscovery
but using it for particular things:
    
    from setuputils import autodiscover_packages, setup_vars
    
    name = "myprog"
    
    description = "My Python Program"
    
    packages = autodiscover_packages(globals())
    
    if __name__ == '__main__':
        from distutils.core import setup
        setup(**setup_vars(globals(), autodiscover=False))

See the docstrings of the individual functions below for more
information.
"""

import os
from datetime import datetime
from distutils.core import Extension
from functools import partial
from itertools import chain, dropwhile, islice, izip, takewhile


def _rst_header_lines(template, basename, **kwds):
    # Generate header lines from template
    if template:
        return [
            "{}{}".format(line, os.linesep) for line in
            template.format(basename=basename, **kwds).splitlines()
        ]
    return []


def _rst(basename, template, startline=None, rstext="rst", **kwds):
    # Add header to file basename.rst and save to file basename
    with open("{}.{}".format(basename, rstext), 'rU') as f:
        lines = f.readlines()
    outlines = _rst_header_lines(template, basename, **kwds) + lines[startline:]
    with open(basename, 'w') as f:
        f.writelines(outlines)


def convert_rst(template=None,
                dirname=".", rstnames=None, rstext="rst",
                **kwds):
    if rstnames is None:
        rstnames = [
            os.path.splitext(basename)[0]
            for basename in os.listdir(dirname)
            if os.path.splitext(basename)[1] == ".{}".format(rstext)
        ]
    for basename in rstnames:
        _rst(basename, template=template, rstext=rstext, **kwds)


def underline(line, underline='~'):
    """Return an underline string for ``line``.
    
    Intended for use with level 3 and lower headings.
    
    Note that the default for ``underline`` assumes that the level
    1 and 2 underlines are the standard ``=`` and ``-``. You can
    pass a different underline character to this function if needed.
    """
    
    if line.endswith('\n'):
        line = line[:-1]
    return "{}\n".format(underline * len(line))


h3_prefix = '### '


def _rst_from_md(basename, template=None, startline=None, mdext="md", **kwds):
    # Convert Markdown text from file named basename.md to RST
    # text saved to file named basename
    with open("{}.{}".format(basename, mdext), 'rU') as f:
        lines = f.readlines()
    outlines = []
    # Allow adding a header to the RST version (which can
    # be filtered out when generating PKG_INFO if desired)
    outlines.extend(_rst_header_lines(template, basename, **kwds))
    # Normally you will want to pass an integer in as the
    # startline keyword argument, since the Markdown file
    # will probably have a title at the start, and the RST
    # files used to provide PKG_INFO to PyPI should not have
    # title lines (since PyPI adds a title itself)
    for line in islice(lines, startline, None):
        is_h3line = line.startswith(h3_prefix)
        if is_h3line:
            line = line[len(h3_prefix):]
        outlines.append(line)
        if is_h3line:
            outlines.append(underline(line))
    with open(basename, 'w') as f:
        f.writelines(outlines)


def convert_md_to_rst(template=None,
                      dirname=".", mdnames=None, mdext="md",
                      **kwds):
    """Convert Markdown text files to RST text files.
    
    Useful for projects where "official" releases are on PyPI but
    development source is visible somewhere else that uses Markdown
    instead of ReStructuredText as its default text doc format,
    such as github. The ``mdnames`` parameter gives the base names
    of files to be converted; this defaults to every text file in
    directory ``dirname`` (which defaults to the directory your
    setup script is run from) with an ``.md`` extension.
    """
    
    if mdnames is None:
        mdnames = [
            os.path.splitext(basename)[0]
            for basename in os.listdir(dirname)
            if os.path.splitext(basename)[1] == ".{}".format(mdext)
        ]
    for basename in mdnames:
        _rst_from_md(basename, template=template, mdext=mdext, **kwds)


def current_date(fmt):
    return datetime.now().strftime(fmt)


def long_description(varmap, filename="README",
                     startline=None, startspec=None,
                     endline=None, endspec=None):
    """Extract long description suitable for PyPI from RST text file.
    
    Useful to automate generating the ``long_description``
    setup parameter. Looks in ``filename``, which should be
    a ReStructuredText file, and returns all the text from
    the given starting line (which is included in the result)
    to the given ending line (which is not included in the
    result). Lines can be specified either by line number
    (``startline`` or ``endline``) or a string the line
    starts with (``startspec`` or ``endspec``). Defaults
    to using the ``README`` file.
    """
    
    startline = startline or varmap.get('startline')
    startspec = startspec or varmap.get('startspec')
    endline = endline or varmap.get('endline')
    endspec = endspec or varmap.get('endspec')
    
    with open(filename, 'rU') as f:
        lines = f.readlines()
    
    if startline or endline:
        innerlines = islice(lines, startline, endline)
    else:
        innerlines = lines
    if startspec:
        inner = dropwhile(
            lambda line: not line.startswith(startspec),
            innerlines
        )
    else:
        inner = innerlines
    if endspec:
        desclines = takewhile(
            lambda line: not line.startswith(endspec),
            inner
        )
    else:
        desclines = inner
    
    return "".join(desclines)


def pypi_url(varmap):
    """Return the PyPI URL for program ``name`` from the setup vars.
    """
    return "http://pypi.python.org/pypi/{}".format(varmap['name'])


def provides(varmap):
    """Return a minimal ``provides`` list from the setup vars.
    """
    
    try:
        return ["{} ({})".format(varmap['name'], varmap['version'])]
    except KeyError:
        return [varmap['name']]


def add_vars(varmap):
    """Automate adding ``long_description``, ``url``, and ``provides`` setup vars.
    """
    
    if 'long_description' not in varmap:
        varmap['long_description'] = long_description(varmap)
    if 'url' not in varmap:
        varmap['url'] = pypi_url(varmap)
    if 'provides' not in varmap:
        varmap['provides'] = provides(varmap)


def convert_lists(varmap,
                  listnames=('classifiers', 'requires', 'provides', 'obsoletes')):
    """Convert long strings to lists of strings.
    
    Allows variable names in ``listnames`` to be specified as
    long strings instead of lists, for easier typing. The
    ``license_map`` and ``devstatus_trove`` strings below give
    examples of the kind of long strings that can be used.
    """
    
    listnames = varmap.get('listnames', listnames)
    for listname in listnames:
        try:
            var = varmap[listname]
        except KeyError:
            pass
        else:
            if isinstance(var, basestring):
                varmap[listname] = var.strip().splitlines()


def add_classifier_python(varmap):
    """Automate adding the Python language classifier.
    
    Since most programs using this module will be Python
    programs, this makes it easy to ensure that the Python
    language Trove classifier is present.
    """
    
    classifiers = varmap.setdefault('classifiers', [])
    if all(not c.startswith("Programming Language ::") for c in classifiers):
        classifiers.append("Programming Language :: Python")


license_map = dict(izip("""
AFL
Apache
BSD
AGPLv3
AGPLv3+
FDL
GPL
GPLv2
GPLv2+
GPLv3
GPLv3+
LGPLv2
LGPLv2+
LGPLv3
LGPLv3+
LGPL
MIT
MPL
MPL 1.1
MPL 2.0
CNRI
PSF
QPL
""".strip().splitlines(),"""
License :: OSI Approved :: Academic Free License (AFL)
License :: OSI Approved :: Apache Software License
License :: OSI Approved :: BSD License
License :: OSI Approved :: GNU Affero General Public License v3
License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)
License :: OSI Approved :: GNU Free Documentation License (FDL)
License :: OSI Approved :: GNU General Public License (GPL)
License :: OSI Approved :: GNU General Public License v2 (GPLv2)
License :: OSI Approved :: GNU General Public License v2 or later (GPLv2+)
License :: OSI Approved :: GNU General Public License v3 (GPLv3)
License :: OSI Approved :: GNU General Public License v3 or later (GPLv3+)
License :: OSI Approved :: GNU Lesser General Public License v2 (LGPLv2)
License :: OSI Approved :: GNU Lesser General Public License v2 or later (LGPLv2+)
License :: OSI Approved :: GNU Lesser General Public License v3 (LGPLv3)
License :: OSI Approved :: GNU Lesser General Public License v3 or later (LGPLv3+)
License :: OSI Approved :: GNU Library or Lesser General Public License (LGPL)
License :: OSI Approved :: MIT License
License :: OSI Approved :: Mozilla Public License 1.0 (MPL)
License :: OSI Approved :: Mozilla Public License 1.1 (MPL 1.1)
License :: OSI Approved :: Mozilla Public License 2.0 (MPL 2.0)
License :: OSI Approved :: Python License (CNRI Python License)
License :: OSI Approved :: Python Software Foundation License
License :: OSI Approved :: Qt Public License (QPL)
""".strip().splitlines()))


def add_classifier_license(varmap):
    """Convert ``license`` setup var to Trove classifier.
    
    Allows short specification of common licenses while still
    adhering to the PyPI spec, which says that the ``license``
    setup keyword should only be used for licenses that don't
    have a Trove classifier.
    """
    
    try:
        c = license_map[varmap['license']]
    except KeyError:
        pass
    else:
        varmap.setdefault('classifiers', []).append(c)
        # PyPI docs say the "License" keyword should only be used for
        # licenses that don't have a Trove classifier
        del varmap['license']


devstatus_trove = """
Development Status :: 3 - Alpha
Development Status :: 4 - Beta
Development Status :: 5 - Production/Stable
Development Status :: 6 - Mature
Development Status :: 7 - Inactive
Development Status :: 1 - Planning
Development Status :: 2 - Pre-Alpha
""".strip().splitlines()


def add_classifier_dev_status(varmap):
    """Convert ``dev_status`` setup var to Trove classifier.
    
    Allows short specification of development status instead
    of having to type the full Trove classifier.
    """
    
    try:
        dev_status = varmap['dev_status']
    except KeyError:
        pass
    else:
        for c in devstatus_trove:
            if c.endswith(dev_status):
                varmap.setdefault('classifiers', []).append(c)
                break


def add_classifiers(varmap):
    """Automate adding standard classifiers to setup vars.
    
    Also sorts the list of classifiers for neatness.
    """
    
    add_classifier_python(varmap)
    add_classifier_license(varmap)
    add_classifier_dev_status(varmap)
    varmap['classifiers'] = sorted(varmap['classifiers'])


def package_root(varmap):
    """Return correct directory for the root package.
    
    The "root" package is the one where modules which do not
    belong to a package are located (see the distutils docs).
    This function should not need to be called directly in
    your actual setup script, but it can be useful for testing.
    Note that it looks for a ``package_root`` setup var, which
    can be used as a shortcut if your "root" package is not in
    the standard place (which is your setup script directory),
    rather than having to declare a ``package_dir`` dictionary
    with a ``''`` key.
    """
    
    try:
        return varmap['package_dir']['']
    except KeyError:
        try:
            result = varmap['package_root']
        except KeyError:
            return "."
        else:
            dirmap = varmap.setdefault('package_dir', {})
            if '' not in dirmap:
                dirmap[''] = result
            return result


def autodiscover_modules(varmap, excludes=("setup", "setuputils")):
    """Return list of Python modules in your distro.
    
    Module names listed in ``excludes`` will not be included; by
    default only ``setup.py`` and ``setuputils.py`` are excluded,
    all other ``.py`` files in your "root" package are included.
    """
    
    excludes = varmap.get('mod_excludes', excludes)
    return [
        os.path.splitext(filename)[0]
        for filename in os.listdir(package_root(varmap))
        if (os.path.splitext(filename)[1] == ".py")
        and (os.path.splitext(filename)[0] not in excludes)
    ]


def add_py_modules(varmap):
    """Automatically fill in the ``py_modules`` setup var.
    """
    
    if 'py_modules' not in varmap:
        varmap['py_modules'] = autodiscover_modules(varmap)


def dir_to_package(dirname):
    return dirname.replace(os.sep, '.')


def package_paths(varmap):
    """Return list of directories in your distro that are Python packages.
    
    This function should not need to be called directly in
    your actual setup script, but it can be useful for testing.
    It uses the "root" package, as determined from your setup
    vars, to determine where to start the search.
    """
    
    rootdir = package_root(varmap)
    result = []
    for dirname, subdirs, filenames in os.walk(rootdir):
        if dirname != rootdir:
            if "__init__.py" in filenames:
                result.append(dirname.split(os.sep, 1)[1])
            else:
                # Don't recurse into subdirs in non-packages
                subdirs[:] = []
    return result


def autodiscover_packages(varmap):
    """Return list of Python packages in your distro.
    """
    
    return [
        dir_to_package(dirname)
        for dirname in package_paths(varmap)
    ]


def add_packages(varmap):
    """Automatically fill in the ``packages`` setup var.
    """
    
    if 'packages' not in varmap:
        varmap['packages'] = autodiscover_packages(varmap)


def _package_data_paths(pathname, ext_srcdir):
    # Return list of package subdirectories that are not packages
    # themselves (and are therefore presumed to contain package data)
    return [
        "{}/*.*".format(subdir)
        for subdir in os.listdir(pathname)
        if (subdir != "__pycache__")
        and (subdir != ext_srcdir)
        and os.path.isdir(os.path.join(pathname, subdir))
        and ("__init__.py" not in os.listdir(os.path.join(pathname, subdir)))
    ]


def autodiscover_package_data(varmap, ext_srcdir=""):
    """Return mapping of package data paths to file lists.
    
    The ``ext_srcdir`` argument is used to exclude package
    subdirectories (if any) that contain extension source files.
    Leaving it blank (the default) means all subdirectories of
    all packages are checked.
    """
    
    ext_srcdir = varmap.get('ext_srcdir', ext_srcdir)
    return dict(
        (dir_to_package(pathname), _package_data_paths(pathname, ext_srcdir))
        for pathname in package_paths(varmap)
        if _package_data_paths(pathname, ext_srcdir)
    )


def add_package_data(varmap):
    """Automatically fill in the ``package_data`` setup var.
    """
    
    if 'package_data' not in varmap:
        varmap['package_data'] = autodiscover_package_data(varmap)


def autodiscover_extensions(varmap,
                            ext_srcdir="",
                            ext_exts=(".c", ".cc", ".cpp", ".i")):
    """Return list of ``Extension`` instances for your distro.
    
    Looks at the ``ext_names`` setup var for the names of
    extension modules (dotted names indicate extensions that
    live inside packages). Each extension name is converted
    to a path relative to your "root" package in which the
    extension is located.
    
    The ``ext_srcdir`` argument, if non-blank, indicates that
    extension source files are in a subdirectory of the extension
    directory (for example, the ``src`` subdirectory).
    
    The ``ext_exts`` argument gives the file extensions for source
    files (the default should work for most cases).
    
    Note that this function assumes that only one extension
    "lives" in a given directory; multiple extensions in the
    same directory can't be autodiscovered using this mechanism.
    """
    
    rootdir = package_root(varmap)
    result = []
    try:
        extnames = varmap['ext_names']
    except KeyError:
        pass
    else:
        ext_srcdir = varmap.get('ext_srcdir', ext_srcdir)
        ext_exts = varmap.get('ext_exts', ext_exts)
        for extname in extnames:
            srcpath = os.path.join(
                rootdir,
                os.path.dirname(extname.replace('.', os.sep))
            )
            if ext_srcdir:
                srcpath = os.path.join(srcpath, ext_srcdir)
            sources = [
                "{}/{}".format(srcpath, basename)
                for basename in os.listdir(srcpath)
                if os.path.splitext(basename)[1] in ext_exts
            ]
            result.append(
                Extension(extname, sources)
            )
    return result


def add_extensions(varmap):
    """Automatically fill in the ``ext_modules`` setup var.
    """
    
    if 'ext_modules' not in varmap:
        varmap['ext_modules'] = autodiscover_extensions(varmap)


def autodiscover_datafiles(varmap):
    """Return list of (dist directory, data file list) 2-tuples.
    
    The ``data_dirs`` setup var is used to give a list of
    subdirectories in your source distro that contain data
    files. It is assumed that all such files will go in the
    ``share`` subdirectory of the prefix where distutils is
    installing your distro (see the distutils docs); within
    that directory, a subdirectory with the same name as
    your program (i.e., the ``name`` setup var) will be
    created, and each directory in ``data_dirs`` will be a
    subdirectory of that. So, for example, if you have example
    programs using your distro in the ``"examples"`` directory
    in your distro, you would declare ``data_dirs = "examples"``
    in your setup vars, and everything under that source
    directory would be installed into ``share/myprog/examples``.
    """
    
    result = []
    try:
        datadirs = varmap['data_dirs']
    except KeyError:
        pass
    else:
        pathprefix = "share/{}".format(varmap['name'])
        for datadir in datadirs:
            for dirname, subdirs, filenames in os.walk(datadir):
                if filenames and ("." not in dirname):
                    distdir = dirname.replace(os.sep, '/')
                    distfiles = [
                        "{}/{}".format(distdir, filename)
                        for filename in filenames
                        if not filename.startswith(".")
                    ]
                    if distfiles:
                        distdir = dirname.replace(os.sep, '/')
                        result.append(
                            ("{}/{}".format(pathprefix, distdir), distfiles)
                        )
    return result


def add_datafiles(varmap):
    """Automatically fill in the ``data_files`` setup var.
    """
    
    if 'data_files' not in varmap:
        varmap['data_files'] = autodiscover_datafiles(varmap)


def autodiscover_scripts(varmap, dirname="scripts"):
    """Return a list of scripts in your distro.
    
    The ``dirname`` argument gives the directory in your source
    distro in which to look for scripts. The ``script_dir``
    setup var can be used to customize this if for some strange
    reason you can't use the default of ``"scripts"``.
    """
    
    dirname = varmap.get('script_dir', dirname)
    if os.path.isdir(dirname):
        return [
            os.path.join(dirname, filename)
            for filename in os.listdir(dirname)
        ]
    return []


def add_scripts(varmap):
    """Automatically fill in the ``scripts`` setup var.
    """
    
    if 'scripts' not in varmap:
        varmap['scripts'] = autodiscover_scripts(varmap)


def autodiscover_all(varmap):
    """Automatically fill in all auto-discovered setup variables.
    
    Note that even if you don't use all of these variables
    (for example, your distro may have only packages and no
    py_modules, or you may have no package data, extensions,
    etc.), you can still use this function; if it finds no
    instances of a given item, there is no effect.
    """
    
    add_py_modules(varmap)
    add_packages(varmap)
    add_package_data(varmap)
    add_extensions(varmap)
    add_datafiles(varmap)
    add_scripts(varmap)


def _add_package_data_lines(pdata, lines):
    # Distutils won't automatically include package_data
    # if we have a MANIFEST.in, so we need to include it
    # in MANIFEST.in (actually, there is supposedly a
    # fix for that that's in the head of the Python source
    # tree, but it isn't in all recent versions and
    # distutils will take care of any duplicate file
    # inclusions anyway, so it doesn't hurt to make sure
    # with the "belt and suspenders" approach here)
    for pkgname, items in pdata.iteritems():
        pkgname = pkgname.replace('.', '/')
        for item in items:
            if '/' in item:
                dirname, filespec = item.rsplit('/', 1)
                dirname = "{}/{}".format(pkgname, dirname)
            else:
                dirname = pkgname
                filespec = item
            lines.append(
                "recursive-include {} {}\n".format(dirname, filespec)
            )


def _add_data_dirs_lines(datadirs, lines):
    # Shortcut if data_dirs was specified (in which
    # case we don't need to individually include each
    # file in data_files)
    for datadir in datadirs:
        lines.append(
            "recursive-include {} *.*\n".format(datadir)
        )


def _add_data_files_lines(datafiles, lines):
    # Distutils won't automatically include data_files
    # either if there's a MANIFEST.in (see note above)
    for _, filenames in datafiles:
        for datafile in filenames:
            lines.append(
                "include {}\n".format(datafile)
            )


def _add_scripts_lines(scripts, lines):
    # Same "belt and suspenders" strategy as for package
    # data and data files above; distutils seems to be more
    # consistent about including scripts but...
    for script in scripts:
        lines.append(
            "include {}\n".format(script)
        )


def _add_lines(varmap, key, lines):
    # Factored out for easier use in make_manifest_in
    try:
        data = varmap[key]
    except KeyError:
        return False
    else:
        globals()['_add_{}_lines'.format(key)](data, lines)
        return True


def make_manifest_in(varmap):
    """Automatically generate ``MANIFEST.in`` template.
    
    Most of this can be done automatically based on other
    setup vars, but for any items that are not covered by
    that, you can add a ``MANIFEST.in.in`` template that
    declares those items. That template will occur last in
    the generated ``MANIFEST.in`` file, so it can also be
    used, if necessary, to override any of the automatically
    generated items.
    
    Note that ``setuputils.py`` is automatically included,
    since it is assumed it should be treated the same as
    ``setup.py``.
    
    Also note that files implied by the ``package_data``,
    ``data_files`` (or ``data_dirs`` if ``data_files`` is
    not present), and ``scripts`` setup vars are also included
    in the generated ``MANIFEST.in``. The Python distutils
    are not completely consistent in including these files
    if a ``MANIFEST.in`` template is used, so we make sure
    by including them here (the distutils automatically
    ignore duplicate file specs so there is no harm done
    either way).
    """
    
    lines = ["include setuputils.py\n"]
    
    # Add lines for things that distutils doesn't always
    # add automatically (see comments in subfunctions above)
    for key in ('package_data', ('data_dirs', 'data_files'), 'scripts'):
        if isinstance(key, tuple):
            for k in key:
                if _add_lines(varmap, k, lines):
                    break
        else:
            _add_lines(varmap, key, lines)
    
    # This ensures that pyc files are left out in case the source
    # tree has them from testing
    lines.append("recursive-exclude . *.pyc\n")
    
    # Read from the in.in file last so the user can override
    # any of the above (shouldn't need to but just in case)
    try:
        with open("MANIFEST.in.in", 'rU') as f:
            in_in_lines = f.readlines()
    except IOError:
        pass
    else:
        lines.extend(in_in_lines)
    
    with open("MANIFEST.in", 'w') as f:
        f.writelines(lines)


# The type values in this dict are not currently used, but
# may be used in future

distutils_keywords = dict(
    name=str,
    version=str,
    description=str,
    long_description=str,
    author=str,
    author_email=str,
    maintainer=str,
    maintainer_email=str,
    url=str,
    download_url=str,
    packages=list,
    py_modules=list,
    scripts=list,
    ext_modules=list,
    classifiers=list,
    distclass=type,
    script_name=str,
    script_args=list,
    options=dict,
    license=str,
    keywords=(str, list),
    platforms=(str, list),
    cmdclass=dict,
    data_files=list,
    package_dir=str,
    package_data=dict,
    requires=list,
    provides=list,
    obsoletes=list
)


def setup_vars(varmap, autodiscover=True, force_manifest_in=False):
    """Return dict of keyword arguments for distutils setup function.
    
    Normally the ``varmap`` argument will be the ``globals()``
    of your setup script. The dictionary returned by this function
    is filtered to only include valid setup arguments, so you can
    safely pass your module globals without worrying about extra
    variables declared there.
    
    The ``autodiscover`` argument determines whether arguments
    not explicitly declared in ``varmap`` will be auto-discovered
    using the sub-functions above. For many use cases this will
    be sufficient; however, if you want finer control over what
    is auto-discovered, this argument can be set to ``False``
    and the sub-functions can be used individually (or setup
    arguments can be declared by hand).
    
    The ``force_manifest_in`` argument determines whether a
    ``MANIFEST.in`` template is generated even if you do not
    have a ``MANIFEST.in.in`` template. The chief reason for
    doing this would be to ensure that ``setuputils.py`` is
    included in your distributions. However, in most cases it
    is easier to either have a ``MANIFEST.in.in`` template,
    even an empty one, or to hand-generate your own ``MANIFEST``
    file (which is rarely done since it requires you to include
    *everything* by hand). Note that if you write your own
    ``MANIFEST.in`` file (and leave ``force_manifest_in`` at
    its default), you should be aware of the Python distutils'
    inconsistency about including files implied by the
    ``package_data``, ``data_files``, and ``scripts`` setup
    arguments (see docstrings above), and you must also include
    ``setuputils.py`` yourself.
    """
    
    varmap = dict(varmap)  # so we don't mutate the original
    
    if 'name' not in varmap:
        raise ValueError("You must supply a program name.")
    
    if autodiscover:
        add_vars(varmap)
        convert_lists(varmap)
        add_classifiers(varmap)
        autodiscover_all(varmap)
    
    if force_manifest_in or os.path.isfile("MANIFEST.in.in"):
        make_manifest_in(varmap)
    
    return dict(
        (k, v) for k, v in varmap.iteritems()
        if k in distutils_keywords
    )



#!/usr/bin/env python
"""
Module SerialTelnet
Sub-Package IO.CLASSES of Package PLIB
Copyright (C) 2008-2013 by Peter A. Donis

Released under the GNU General Public License, Version 2
See the LICENSE and README files for more information

This module contains the SerialTelnet class. This is a
telnet client that can communicate over a serial port;
it uses the ``SerialPseudoSocket`` class from this
sub-package to add serial device I/O functionality.
"""

import telnetlib

from plib.io.classes import SerialPseudoSocket


class SerialTelnet(telnetlib.Telnet):
    """Telnet client that can use a serial port or a socket.
    
    Use the ``openserial`` method to open a serial device,
    and ``open`` to open a socket. Does not allow any
    parameters to the constructor (so you have to explicitly
    call the appropriate opening method).
    """
    
    def __init__(self):
        telnetlib.Telnet.__init__(self)  # don't allow any constructor params
    
    def openserial(self, devid):
        self.sock = SerialPseudoSocket(devid, blocking_mode=True)
